/*
 * Copyright (c) 2022 Libre Solar Technologies GmbH
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/zephyr.h>
#include <zephyr/device.h>
#include <zephyr/drivers/uart.h>

#include <string.h>

#define MSG_SIZE 255

/* queue to store up to 10 messages (aligned to 4-byte boundary) */
K_MSGQ_DEFINE(uart_msgq, MSG_SIZE, 10, 4);

static const struct device *uart_dev0 = DEVICE_DT_GET(DT_NODELABEL(uart0));
static const struct device *uart_dev1 = DEVICE_DT_GET(DT_NODELABEL(uart1));

/* receive buffer used in UART ISR callback */
static char rx_buf0[MSG_SIZE+1];
static char rx_buf1[MSG_SIZE+1];
static int rx_buf_pos0;
static int rx_buf_pos1;

/*
 * Print a null-terminated string character by character to the UART interface
 */
void print_uart(const struct device *dev, char *buf)
{
	int msg_len = strlen(buf);

	for (int i = 0; i < msg_len; i++) {
		uart_poll_out(dev, buf[i]);
	}
}

/*
 * Read characters from UART until line end is detected. Afterwards push the
 * data to the message queue.
 */
void serial_cb0(const struct device *dev, void *user_data)
{
	uint8_t c;

	if (!uart_irq_update(uart_dev0)) {
		return;
	}

	while (uart_irq_rx_ready(uart_dev0)) {
		//printk("UART0 RX");

		uart_fifo_read(uart_dev0, &c, 1);

		if ((c == '\n' || c == '\r') && rx_buf_pos0 > 0) {
			/* terminate string */
			rx_buf0[rx_buf_pos0] = c;
			rx_buf0[rx_buf_pos0+1] = '\0';

			/* Send received string back to opposite UART */
			print_uart(uart_dev0, rx_buf0);

			/* reset the buffer (it was copied to the msgq) */
			rx_buf_pos0 = 0;
		} else if (rx_buf_pos0 < (sizeof(rx_buf0) - 1)) {
			rx_buf0[rx_buf_pos0++] = c;
		}
		/* else: characters beyond buffer size are dropped */
	}
	
}

/*
 * Read characters from UART until line end is detected. Afterwards push the
 * data to the message queue.
 */
void serial_cb1(const struct device *dev, void *user_data)
{
	uint8_t c;

	if (!uart_irq_update(uart_dev1)) {
		return;
	}

	while (uart_irq_rx_ready(uart_dev1)) {
		printk("UART1 RX");

		uart_fifo_read(uart_dev1, &c, 1);

		if ((c == '\n' || c == '\r') && rx_buf_pos1 > 0) {
			/* terminate string */
			rx_buf1[rx_buf_pos1] = c;
			rx_buf1[rx_buf_pos1+1] = '\0';

			/* Send received string back to opposite UART */
			print_uart(uart_dev1, rx_buf1);

			/* reset the buffer (it was copied to the msgq) */
			rx_buf_pos1 = 0;
		} else if (rx_buf_pos1 < (sizeof(rx_buf1) - 1)) {
			rx_buf1[rx_buf_pos1++] = c;
		}
		/* else: characters beyond buffer size are dropped */
	}
	
}



void main(void)
{

        char tx_buf[MSG_SIZE];
	if (!device_is_ready(uart_dev0)) {
		printk("UART0 device not found!");
		return;
	}
 
	if (!device_is_ready(uart_dev1)) {
		printk("UART1 device not found!");
		return;
	}

	/* configure interrupt and callback to receive data */
	uart_irq_callback_user_data_set(uart_dev0, serial_cb0, NULL);
	uart_irq_rx_enable(uart_dev0);

	uart_irq_callback_user_data_set(uart_dev1, serial_cb1, NULL);
	uart_irq_rx_enable(uart_dev1);

	printk("Two UART sample started!");

	/* indefinitely wait for input from the user */
	while (1) {

	}
}
